/*
  Sketch for operating the HMC6343
 Writen by Shachar Geiger on 12.6.09
 For more information, see http://www.magneticsensors.com/datasheets/HMC6343.pdf
 
 NOTICE: This is an expansive sensor! do not connect to power until after verifying the connections!
 
 Connect the sensor using a logic level converter (from Sparkfun):
 Sensor VCC -> Arduino 3V3
 Sensor GND -> Arduino Gnd
 Sensor SDA -> Converter TXI (Chan1)
 Converter TXO (Chan1) -> Arduino ANALOG IN 4
 Sensor SCL -> Converter TXI (Chan2)
 Converter TXO (Chan2) -> Arduino ANALOG IN 5
 Converter GND (at least one of them) -> Arduino Gnd
 Converter HV -> Arduino 5V
 Converter LV -> Arduino 3V3
 
 The sketch communicates with the sensor, enabling the following:
 Set the sample rate to 5Hz or 10Hz (Readings per second).
 Read the heading information (Heading, Pitch, Roll)
 Read the accellerations information (X, Y, Z)
 Read the magnetics information (X, Y, Z)
 
 Each reading procedurre gets the information from the sensor and stores it into the 3 variables: x, y and z.
 */

#include <Wire.h>
#define compass_ADR 0x32>>1

byte data[6];
int x;
int y;
int z;
int detect;

void set5Hz() {
  Wire.beginTransmission(compass_ADR);
  Wire.send(0xF1);
  Wire.send(0x05);
  Wire.send(0x01);
  Wire.endTransmission();
  delay(10);
}

void set10Hz() {
  Wire.beginTransmission(compass_ADR);
  Wire.send(0xF1);
  Wire.send(0x05);
  Wire.send(0x02);
  Wire.endTransmission();
  delay(10);
}

void getHeading() {
  Wire.beginTransmission(compass_ADR);
  Wire.send(0x50);
  Wire.endTransmission();
  delay(1);
  Wire.requestFrom(compass_ADR, 6);
  for (int i=0; i<6; i++) {
    while (Wire.available() == 0);
    data[i] = Wire.receive();
  }
  x = data[0]*256 + data[1];
  y = data[2]*256 + data[3];
  z = data[4]*256 + data[5];
}

void getAccelerations() {
  Wire.beginTransmission(compass_ADR);
  Wire.send(0x40);
  Wire.endTransmission();
  delay(1);
  Wire.requestFrom(compass_ADR, 6);
  for (int i=0; i<6; i++) {
    while (Wire.available() == 0);
    data[i] = Wire.receive();
  }
  x = data[0]*256 + data[1];
  y = data[2]*256 + data[3];
  z = data[4]*256 + data[5];
}

void getMagnetics() {
  Wire.beginTransmission(compass_ADR);
  Wire.send(0x45);
  Wire.endTransmission();
  delay(1);
  Wire.requestFrom(compass_ADR, 6);
  for (int i=0; i<6; i++) {
    while (Wire.available() == 0);
    data[i] = Wire.receive();
  }
  x = data[0]*256 + data[1];
  y = data[2]*256 + data[3];
  z = data[4]*256 + data[5];
}

void setup() {
  pinMode (11, INPUT);
  pinMode (13, OUTPUT);
  pinMode (7, OUTPUT);
  Wire.begin();
  Serial.begin(9600);
  delay(500);
  set10Hz();
}

void loop() {
  digitalWrite (7, HIGH);
  detect = digitalRead (11);
  if (detect) {
    digitalWrite (13, LOW);
  } 
  else {
    digitalWrite (13, HIGH);
    getHeading();
    Serial.write ('x');
    x = (x+1800) / 14;
    Serial.write(x);
    Serial.write('y');
    y = (y+1800) / 14;
    Serial.write(y);
    Serial.write('z');
    z= (z+1800) / 14;
    Serial.write(z);
    delay(100);
  }
}
